package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * Declares that the parameter, return value, or field to which this annotation
 * is applied is a reference to an object. In particular, its value will not be
 * {@code null}.
 * 
 * @see Nullable
 * @see Raw
 */
@Documented
@Target({ ElementType.FIELD, ElementType.METHOD, ElementType.PARAMETER })
public @interface NonNull {
	/**
	 * When annotating a method, this attribute is used to disambiguate that the
	 * annotation refers to the method's return value, not the method's
	 * receiver. This is not strictly necessary for this annotation, because
	 * targeting the method's receiver unnecessary because it can never be
	 * {@code null}, however, this approach keeps this annotation consistent
	 * with other annotations (such as {@link Unique}).
	 * <p>
	 * If the value is the empty string when annotating a method, a modeling
	 * error is reported.
	 * <p>
	 * A value of {@code "return"} indicates that the return value of the
	 * annotated method is never {@code null}.
	 * <p>
	 * This attribute is not used when annotating a parameter or a field: the
	 * attribute value must be the empty string in these cases.
	 * <p>
	 * The value of this attribute must conform to the following grammar (in <a
	 * href="http://www.ietf.org/rfc/rfc4234.txt">Augmented Backus&ndash;Naur
	 * Form</a>):
	 * 
	 * <pre>
	 * value = [&quot;return&quot;] ; See above comments
	 * </pre>
	 */
	String value() default "";
}
